#################################################################################
# Created       : Martin Foster
# Modified      : 05-May-2003
#################################################################################
#
# Param - Modules part of Ethereal Realms, designed to handle parameters
#         diffrently
# Copyright (C) 2000-2004  Martin Foster
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
# Author of this script can be contacted at the following:
#       E-Mail  : martin@ethereal-realms.org
#       Address : 29-26503 TWP RD 511
#                 Spruce Grove, Alberta
#                 T7Y 1G4
#
#################################################################################

package eRealms::Param;						# Pacakge name

use eRealms::Crypto;						# Cryptographic Module

#################################################################################
# Sub-Routines
#################################################################################

#####################
# Constructor
#
# Allows to class to be exported and thus spawn objects as a result

sub new
{
	####################
	# Data members
	my $class    = shift;					# Class itself
	my $database = shift;					# Database handle
	my $cgi      = shift;					# CGI handle

	my $self;						# Self referncing link


	####################
	# Program area

	# Define parameters
	$self = {
	 XCGI  => $cgi,
	 XDAT  => $database,
	};

	# Bless $self into designated class
	bless $self, $class;

	# Return self
	return $self;
}


#####################
# Cleanup
#
# This function will allow one to clear parameters in the list
# stored within this object.  Essentially it allows to remove based
# on simple keywords.  Much cleaner

sub Cleanup
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $key;						# Parameter key


	####################
	# Program area

	# Cycle and remove
	while ($key = shift(@_))
	{
		delete $self->{$key};
	}
}

#####################
# Crypt
#
# Returns the straight encrypted line instead of the token like system put into place.
# This is used for scripts that need not worry about things like MORI, registration
# and such will benefit from this.

sub Crypt
{
	#####################
	# Data members
	my $self     = shift;					# Self reference

	my $crypt;						# Crytographic handler
	my $param;						# Parameter to embed
	my $key;						# Singular parameter key
	my $val;						# Value

	my %param;						# Parameter hash


	#####################
	# Program Area

	# Holding hash
	while ($key = shift)
	{
		# Pull value
		$val = shift;

		# Popupulate
		$param{$key} = $val if (defined($key));
	}

	# Retreive encrypted string
	$crypt = new eRealms::Crypto;
	$param = $crypt->EncryptParam(%param);

	# Return formatted string
	return "\?SEC\=$param";
}



#####################
# Embed Information
#
# This function will embed retreived parameters for the next pass.    By making use of
# the Cypto module the use of text mangling and unmangling is no longer needed.

sub Embed
{
	#####################
	# Data members
	my $self     = shift;					# Self reference
	my $choice   = shift;					# Choice of the matter

	my $cgi      = $self->{'XCGI'};				# CGI handle
	my $database = $self->{'XDAT'};				# Database handle

	my $crypt;						# Crytographic handler
	my $param;						# Parameter to embed
	my $rand;						# Random value
	my $seq;						# Sequence number
	my $string;						# Encrypted parameter string
	my $user;						# User login name

	my $key;						# Singular parameter key
	my $val;						# Value

        my @keys = qw(ROOM COMM MAIN);				# Re-occuring info

	my %param;						# Parameter hash


	#####################
	# Program Area

	# Legacy random
	srand();

	# Holding hash
	while ($key = shift)
	{
		# Pull value
		$val = shift;

		# Popupulate
		$param{$key} = $val if (defined($key));
	}

	# Value set
	$user = (defined($param{'USER'})) ? $param{'USER'} : 'system';


	# Is this random number used
	do
	{
		# Determin random number
		$rand = int(rand() * 8000000);

		# Verify for existence
		($seq) = $database->DataGetSequence($user, $rand);

	} while (defined($seq));


	# Create embded
	# Determin style
	if ($choice < 1)
	{
		# Embeded parameter list
		$string = $cgi->hidden('XLOG', $user) . " " . $cgi->hidden('SEC', $rand);

		# Possible additions
		foreach $key (@keys)
		{
			# Key exists add
			if (defined($param{$key}))
			{
				# Append to string
				$string = $string . " " . $cgi->hidden($key, $param{$key});

				# Delete key
				delete $param{$key};
			}
		}
	}
	else
	{
		# Escaped
		$val = $cgi->escape($user);

		# Inline parameter lists
		$string = "\?XLOG\=$val\&SEC\=$rand";

		# Possible additions
		foreach $key (@keys)
		{
			# Key exists add
			if (defined($param{$key}))
			{
				# Escaped
				$val = $cgi->escape($param{$key});

				# Append to string
				$string = $string . "\&$key\=$val";

				# Delete key
				delete $param{$key};
			}
		}
	}


	# Retreive encrypted string
	$crypt = new eRealms::Crypto;
	$param = $crypt->EncryptParam(%param, ADDR=>$self->{'XADDR'}, MORI=>$time);

	# Database entrance
	$database->Quick("INSERT INTO Param
		(ParamNumber,
		 PuppeteerLogin,
		 ParamValue)
		VALUES (?,?,?)",
	 $rand,
	 $user,
	 $param);	 	

	# Return value
	return $string;
}

# Alias for Normal
sub EmbedNormal
{ return Embed(shift, 0, @_); }

# Alias for Inline
sub EmbedInline
{ return Embed(shift, 1, @_); }


#####################
# Flatten
#
# As a way to avoid the extreme annoyance of the parameter system dying out, this
# sub-routine will attempt to send all necessary parameters to be embeded.

sub Flat
{
	#####################
	# Data members
	my $self     = shift;					# Self reference

	my $key;						# Singular parameter key
	my %param;						# Parameter hash


	#####################
	# Data members

	# Build holding hash
	foreach $key (keys(%{$self}))
	{
		# Skips
		next if ($key eq 'XCGI');
		next if ($key eq 'XDAT');
		next if ($key eq 'XADDR');
		next if ($key !~ /^[A-Z]+$/);

		# Insert
		$param{$key} = $self->{$key};
	}


	# Return flat hash
	return %param;
}



#####################
# GetParam
#
# Due to the annoyance of having to retrieve parameters from the $cgi->param
# command, I have opted to move them directly into a hash.  Passable to functions
# with little or no trouble and generally avoiding having the same value 
# re-applied to the parameter, causing problems for changes and easier calling
# via a hash.

sub GetParam
{
	#####################
	# Data members	
	my $self     = shift;					# Reference to self

	my $cgi      = $self->{'XCGI'};				# Common gateway interface
	my $database = $self->{'XDAT'};				# Database handle

	my $crypt;						# Cryptographic link
	my $param;						# Specific parameter
	my $string;						# String of parameters

	my $addr;						# Captured address
	my $pked;						# Packed address

	my @names;						# Array of named parameters reveived
	my @vals;						# Array of pulled values


	#####################
	# Program area

	# Security measure
	$cgi->delete('ADDR');
	$cgi->delete('CRYPT');
	$cgi->delete('MORI');

	# Initial pull
	$string = $cgi->param('SEC');

	# Database handler
	if ((defined($string)) 
	 && ($string =~ /^\d+$/))
	{
		# Database specific
		($string) = $database->DataGetParam(
			$cgi->param('XLOG'), 
			$cgi->param('SEC')
		 ); 

		# Cleanup
		$cgi->delete('XLOG');
	}

	# Decrypt and add to hash
	if (defined($string))
	{
		# Initialize handle
		$crypt  = new eRealms::Crypto(); 

		# Create and decrypt
		$crypt->DecryptParam($string, $self);

		# Precautionary measures
		$cgi->delete('SEC');
	}

	# Get list of all parameters received
	@names = $cgi->param();	

	# Cycle though parameter list
	while ($param = shift(@names))
	{
		# Parameter pull
		# Retreive into a holding array
		@vals = $cgi->param($param);

		# Form elements are preffered
		$self->{$param} = pop(@vals);


		# Cleanup
		# Delete
		$cgi->delete($param);

		# Empty
		@vals = [];
	}


	# Capture address
	$addr = $cgi->remote_host();

	# Invalid IP address
	if ($addr !~ /\d{1,3}\.\d{1,3}\.\d{1,3}/)
	{
		# Socket library
		use Socket;

		# Retrieve address
		$pked = gethostbyname($addr);

		# Form into human readable
		$addr = inet_ntoa($pked);
	}

	# Assign address
	$self->{'XADDR'} = $addr;
}

1;								# Return true
